@@ -40,7 +40,8 @@ class SurveyElement(dict):
         u"children": list,
         u"itemset": unicode,
         u"choice_filter": unicode,
-        u"autoplay": unicode
+        u"autoplay": unicode,
+        u"flat": lambda: False,
     }
 
     def _default(self):
@@ -139,7 +140,12 @@ class SurveyElement(dict):
         while current_element.parent:
             current_element = current_element.parent
             result = [current_element] + result
-        return result
+        #For some reason the root element has a True flat property...
+        output = [result[0]]
+        for item in result[1:]:
+            if not item.get(u"flat"):
+                output.append(item)
+        return output
 
     def get_root(self):
         return self.get_lineage()[0]
@@ -209,6 +215,22 @@ class SurveyElement(dict):
                         'lang': lang,
                         'text': text
                     }
+            requiredMsg = bind_dict.get(u'jr:requiredMsg')
+            if type(requiredMsg) is dict:
+                for lang, text in requiredMsg.items():
+                    yield {
+                        'path': self._translation_path(u'jr:requiredMsg'),
+                        'lang': lang,
+                        'text': text
+                    }
+            noAppErrorString = bind_dict.get(u'jr:noAppErrorString')
+            if type(noAppErrorString) is dict:
+                for lang, text in noAppErrorString.items():
+                    yield {
+                        'path': self._translation_path(u'jr:noAppErrorString'),
+                        'lang': lang,
+                        'text': text
+                    }
 
         for display_element in [u'label', u'hint']:
             label_or_hint = self[display_element]
@@ -284,16 +306,23 @@ class SurveyElement(dict):
         Return the binding for this survey element.
         """
         survey = self.get_root()
-        d = self.bind.copy()
-        if d:
-            for k, v in d.items():
+        bind_dict = self.bind.copy()
+        if self.get('flat'):
+            # Don't generate bind element for flat groups.
+            return None
+        if bind_dict:
+            for k, v in bind_dict.items():
                 #I think all the binding conversions should be happening on the xls2json side.
                 if hashable(v) and v in self.binding_conversions:
                     v = self.binding_conversions[v]
                 if k == u'jr:constraintMsg' and type(v) is dict:
                     v = "jr:itext('%s')" % self._translation_path(u'jr:constraintMsg')
-                d[k] = survey.insert_xpaths(v)
-            return node(u"bind", nodeset=self.get_xpath(), **d)
+                if k == u'jr:requiredMsg' and type(v) is dict:
+                    v = "jr:itext('%s')" % self._translation_path(u'jr:requiredMsg')
+                if k == u'jr:noAppErrorString' and type(v) is dict:
+                    v = "jr:itext('%s')" % self._translation_path(u'jr:noAppErrorString')
+                bind_dict[k] = survey.insert_xpaths(v)
+            return node(u"bind", nodeset=self.get_xpath(), **bind_dict)
         return None
 
     def xml_bindings(self):
